#include "core.h"

//----------------------------------------------------------------------------
// Global variables
//----------------------------------------------------------------------------
DX8VARS dx8;
D3DFORMAT format;
D3DFORMAT zFormat;
HRESULT hRes;

//----------------------------------------------------------------------------
// InitD3D() : Initializes Direct3D
//----------------------------------------------------------------------------

HRESULT InitD3D(HWND hWnd, WORD width, WORD height, WORD wDepth, WORD wBackBuffers, BOOL bWindowed)
{
    // save screen parameters

    SetWindowPos(hWnd,HWND_TOP,0,0,width,height,SWP_SHOWWINDOW);
    dx8.pD3D = Direct3DCreate8(D3D_SDK_VERSION);

    if (!dx8.pD3D) {
        return E_FAIL;
    }

    D3DPRESENT_PARAMETERS d3dpp;
    ZeroMemory(&d3dpp,sizeof(d3dpp));
 
	if (bWindowed) {
        d3dpp.Windowed=TRUE;
        D3DDISPLAYMODE d3ddm;
        if( FAILED( hRes=dx8.pD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &d3ddm ) ) ) {
            return E_FAIL;
        }
        format=d3dpp.BackBufferFormat=d3ddm.Format; 
    } else {
        d3dpp.Windowed=FALSE;
        switch (wDepth) {
            case 32:
                DXTestFmt(D3DFMT_A8R8G8B8);
                DXTestFmt(D3DFMT_X8R8G8B8);
            case 16:
                DXTestFmt(D3DFMT_R5G6B5);
                DXTestFmt(D3DFMT_X1R5G5B5);
            default:
                format=D3DFMT_UNKNOWN;
        }
        if (format==D3DFMT_UNKNOWN) {
            return E_FAIL;
        }
        d3dpp.BackBufferFormat=format;
        d3dpp.FullScreen_RefreshRateInHz=D3DPRESENT_RATE_DEFAULT;
        d3dpp.FullScreen_PresentationInterval=D3DPRESENT_INTERVAL_ONE;
    }
    d3dpp.BackBufferWidth=width;
    d3dpp.BackBufferHeight=height;
    d3dpp.BackBufferCount=wBackBuffers;
    d3dpp.SwapEffect = D3DSWAPEFFECT_FLIP;

    zFormat=D3DFMT_UNKNOWN;
    DXTestDepth(D3DFMT_D32);
    else DXTestDepth(D3DFMT_D32);
    else DXTestDepth(D3DFMT_D24S8);
    else DXTestDepth(D3DFMT_D24X4S4);
    else DXTestDepth(D3DFMT_D24X8);
    else DXTestDepth(D3DFMT_D16);
    else DXTestDepth(D3DFMT_D15S1);
    else DXTestDepth(D3DFMT_D16_LOCKABLE);
    if (zFormat!=D3DFMT_UNKNOWN) {
        d3dpp.EnableAutoDepthStencil=TRUE;
        d3dpp.AutoDepthStencilFormat=zFormat;
    }
    if( FAILED( hRes=dx8.pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
                                          D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                                          &d3dpp,&dx8.pd3dDevice ) ) ) {

        return E_FAIL;
    }    

	D3DXCreateSprite(dx8.pd3dDevice , &dx8.pd3dxSprite);
    // return success

    return S_OK;
}

BOOL InitDX(WORD wWidth,WORD wHeight,WORD wDepth,WORD wBackBuffers,BOOL bWindowed)
{
	return true;
}

//----------------------------------------------------------------------------
// Cleanup() : Releases all previously initialized objects
//----------------------------------------------------------------------------

void Cleanup()
{
	if( dx8.pd3dDevice ) 
		dx8.pd3dDevice->Release();
	
	if( dx8.pD3D )
		dx8.pD3D->Release();
}

BOOL TestFormat(D3DFORMAT fmt)
{
    if (D3D_OK==dx8.pD3D->CheckDeviceType(D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,fmt,fmt,FALSE))
        return TRUE;
    return FALSE;
}

BOOL TestDepth(D3DFORMAT fmt)
{
    if (D3D_OK!=dx8.pD3D->CheckDeviceFormat(D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,format,
                                         D3DUSAGE_DEPTHSTENCIL,D3DRTYPE_SURFACE,fmt))
        return FALSE;
    if (D3D_OK!=dx8.pD3D->CheckDepthStencilMatch(D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,
                                              format,format,fmt))
        return FALSE;
    return TRUE;
}

//----------------------------------------------------------------------------
// LoadTexture() : Load a texture from a file, returns ptr to texture.
//                   D3DX8 supports BMP, PPM, DDS, JPG, PNG, TGA & DIB. 
//----------------------------------------------------------------------------
LPDIRECT3DTEXTURE8 LoadTexture(char *filename)
{
     LPDIRECT3DTEXTURE8 pd3dTexture;
     // Set black as our source color key. Use 0xFFFF00FF for magenta instead.
     // Use 0x00000000 for no 'color key' substitution
     D3DCOLOR colorkey = 0xFF000000;
     // The D3DX function will fill in the following image info for us 
     D3DXIMAGE_INFO SrcInfo; // Optional 

     // Load image from file - maintain size of original file.
     // It is also possible to specify a new height/width and ask D3DX to      filter
     // the image to the new size (stretch/shrink). See SDK docs for details.
     D3DXCreateTextureFromFileEx( dx8.pd3dDevice, filename, 0, 0, 1, 0, 
          D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, D3DX_FILTER_NONE, D3DX_DEFAULT, 
          colorkey, &SrcInfo , NULL, &pd3dTexture);
     // Check the return value here, etc

     // Remember, textures sizes may be larger or smaller than your image, 
     // to allow for legal texture sizes & maximums on user's hardware
     return pd3dTexture;
}


//----------------------------------------------------------------------------
// BltSprite() : A replacement of the DirectDraw Blt() function for Direct3D8
// Supports DDBLTFX_MIRRORLEFTRIGHT and DDBLTFX_MIRRORUPDOWN
// If pDestRect is NULL, copies it to the same co-ord as src
// If pSrcRect is NULL, copies the entire texture
//----------------------------------------------------------------------------
HRESULT BltSprite( RECT *pDestRect, LPDIRECT3DTEXTURE8 pSrcTexture, 
          RECT *pSrcRect, DWORD dwFlags )
{
     D3DXVECTOR2 scaling(1,1), rcenter(0,0), trans(0,0);

     // Set the parameters - translation (screen location)
     if (pDestRect) {
          trans.x = (float) pDestRect->left;
          trans.y = (float) pDestRect->top;
     } 
     // Scaling - If Source & Destination are different size rects, then scale
     if (pDestRect && pSrcRect) {
          scaling.x = ((float) (pDestRect->right - pDestRect->left)) / 
               ((float) (pSrcRect->right - pSrcRect->left));
          scaling.y = ((float) (pDestRect->bottom - pDestRect->top)) / 
               ((float) (pSrcRect->bottom - pSrcRect->top));
     }
     // Handle the flags - need to change scaling & adjust translation
     if (pSrcRect && dwFlags) {
          if (dwFlags&DDBLTFX_MIRRORLEFTRIGHT) { 
               scaling.x = -scaling.x; 
               trans.x += (float) (pDestRect->right - pDestRect->left); 
          }
          if (dwFlags&DDBLTFX_MIRRORUPDOWN) { 
               scaling.y = -scaling.y; 
               trans.y += (float) (pDestRect->bottom - pDestRect->top); 
          }
     }

     // And here we go:
     return dx8.pd3dxSprite->Draw( pSrcTexture, pSrcRect, &scaling, 
               &rcenter, 0, &trans, 0xFFFFFFFF );

}

// LISTING 4
//----------------------------------------------------------------------------
// BltSpriteEx() : A replacement of the DirectDraw Blt() function for Direct3D8
// Supports DDBLTFX_MIRRORLEFTRIGHT and DDBLTFX_MIRRORUPDOWN,
// modulate is color/alpha modifier
// rotation is value of rotation in RADIANS, counter-clockwise
// prcenter is the rotation centre point of the obj, or NULL
// If pDestRect is NULL, copies it to the same co-ord as src
// If pSrcRect is NULL, copies the entire texture
//----------------------------------------------------------------------------
HRESULT BltSpriteEx( RECT *pDestRect, LPDIRECT3DTEXTURE8 pSrcTexture, 
               RECT *pSrcRect, DWORD dwFlags, D3DCOLOR modulate, 
               float rotation, POINT *prcenter)
{
     D3DXVECTOR2 scaling(1,1), rcenter(0,0), trans(0,0); 

     // Set the parameters - translation (screen location)
     if (pDestRect) {
          trans.x = (float) pDestRect->left;
          trans.y = (float) pDestRect->top;
     } 
     // Rotation Center
     if (prcenter) {
          rcenter.x = (float) prcenter->x;
          rcenter.y = (float) prcenter->y;
     } else if (pSrcRect) {
          // Set Rotation Center to be object center if none provided
          rcenter.x = (float) (pSrcRect->right - pSrcRect->left) / 2;
          rcenter.y = (float) (pSrcRect->bottom - pSrcRect->top) / 2;
     }
     // Scaling - If Source & Destination are different size rects, then scale
     if (pDestRect && pSrcRect) {
          scaling.x = ((float) (pDestRect->right - pDestRect->left)) / 
               ((float) (pSrcRect->right - pSrcRect->left));
          scaling.y = ((float) (pDestRect->bottom - pDestRect->top)) / 
               ((float) (pSrcRect->bottom - pSrcRect->top));
     }
     // Handle the flags - need to change scaling & adjust translation
     if (pSrcRect && dwFlags) {
          if (dwFlags&DDBLTFX_MIRRORLEFTRIGHT) { 
               scaling.x = -scaling.x; 
               trans.x += (float) (pDestRect->right - pDestRect->left); 
     }
          if (dwFlags&DDBLTFX_MIRRORUPDOWN) { 
               scaling.y = -scaling.y; 
               trans.y += (float) (pDestRect->bottom - pDestRect->top); 
          }
     }

     // And here we go:
     return dx8.pd3dxSprite->Draw( pSrcTexture, pSrcRect, &scaling, 
               &rcenter, rotation, &trans, modulate );

}


//-----------------------------------------------------------------------------
// DrawRect() : Draws a rect on 'the viewport' using Clear, with some options
//      This is not a Microsoft-endorsed use of the Clear command!
//      Note that the Alpha values of colors are significant - 0xFF for opaque
//      fillstyle: GDI values of BS_SOLID or BS_NULL (hollow)
//      bordersize is -ve for internal to rectangle
//      drflags are DR3DLOOK or DR3DLOOK2 for embossed shadow button effects
//-----------------------------------------------------------------------------
HRESULT DrawRect(RECT rc, DWORD fillstyle, D3DCOLOR fillcolor, int bordersize, 
          D3DCOLOR bordercolor, DWORD drflags)
{
     // Error if not a normal rectangle
     if (rc.bottom-rc.top < 1) return DDERR_INVALIDPARAMS; 
     if (rc.right-rc.left < 1) return DDERR_INVALIDPARAMS; 

     // First, we do the inside (fill) part
     switch (fillstyle) {
     case BS_SOLID:
          // Fill area is full size of rectangle
          dx8.pd3dDevice->Clear( 1, (D3DRECT *)&rc, D3DCLEAR_TARGET, 
               fillcolor, 1.0f, 0);
          break;
     case BS_NULL:
     default:
          // No fill or unknown, no action
          break;
     }

     // This handles drawing the borders (four sides)
     if (bordersize != 0) {
          D3DCOLOR topbord=bordercolor, botbord=bordercolor;
          RECT brc;
          // Adjust for border outside of rect if +ve border size:
          if (bordersize > 0) {
               rc.left -= bordersize; 
               rc.right += bordersize; 
               rc.top -= bordersize; 
               rc.bottom += bordersize; 
               bordersize = -bordersize; 
          }

          // Embossed effect flags:
          if (drflags & DR3DLOOK) {
               topbord = DRGB( (GetR(bordercolor)+255)/2, 
                    (GetG(bordercolor)+255)/2, (GetB(bordercolor)+255)/2 );
               botbord = DRGB( (GetR(bordercolor))/2, 
                         (GetG(bordercolor))/2, (GetB(bordercolor))/2 );
          } else if (drflags & DR3DLOOK2) {
               // Reversed shadow, for 'selected' look
               topbord = DRGB( (GetR(bordercolor))/2, 
                    (GetG(bordercolor))/2, (GetB(bordercolor))/2 );
               botbord = DRGB( (GetR(bordercolor)+255)/2, 
                    (GetG(bordercolor)+255)/2, (GetB(bordercolor)+255)/2 );
          } 

          // Draw top and left
          SetRect(&brc, rc.left, rc.top, rc.right, rc.top - bordersize);
          dx8.pd3dDevice->Clear( 1, (D3DRECT *)&brc, D3DCLEAR_TARGET, 
                    topbord, 1.0f, 0);
          SetRect(&brc, rc.left, rc.top, rc.left-bordersize, rc.bottom);
          dx8.pd3dDevice->Clear( 1, (D3DRECT *)&brc, D3DCLEAR_TARGET, 
                    topbord, 1.0f, 0);

          // Draw bottom and right
          SetRect(&brc, rc.left, rc.bottom+bordersize, rc.right, rc.bottom);
          dx8.pd3dDevice->Clear( 1, (D3DRECT *)&brc, D3DCLEAR_TARGET, 
                    botbord, 1.0f, 0);
          SetRect(&brc, rc.right+bordersize, rc.top, rc.right, rc.bottom);
          dx8.pd3dDevice->Clear( 1, (D3DRECT *)&brc, D3DCLEAR_TARGET, 
                    botbord, 1.0f, 0);

     }

     return S_OK;
}

